package net.javase.hksup.starter.service;

import com.alibaba.fastjson2.JSONObject;
import com.sun.jna.Native;
import lombok.extern.slf4j.Slf4j;
import net.javase.hksup.starter.config.HkSupSdkProperties;
import net.javase.hksup.starter.model.FaceInfoParam;
import net.javase.hksup.starter.model.SearchFaceInfoParam;
import net.javase.hksup.starter.model.UserInfo;
import net.javase.hksup.starter.model.UserInfoDetail;
import net.javase.hksup.starter.sdk.HCISUPCMS;
import net.javase.hksup.starter.utils.SystemUtil;
import org.springframework.context.ApplicationContext;

import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * CmsServer
 *
 * @author Frank
 */
@Slf4j
public class CmsServer {


    private final HkSupSdkProperties supSdkProperties;

    private final ApplicationContext applicationContext;

    public static HCISUPCMS hciSupCms = null;
    private int CmsHandle = -1; //CMS监听句柄
    private HCISUPCMS.DEVICE_REGISTER_CB deviceRegisterCb;
    HCISUPCMS.NET_EHOME_CMS_LISTEN_PARAM struCMSListenPara = new HCISUPCMS.NET_EHOME_CMS_LISTEN_PARAM();

    private CmsServerCallback callback = null;

    private static final int DEFAULT_TIMEOUT = 5000;

    public CmsServer(HkSupSdkProperties supSdkProperties, ApplicationContext applicationContext) {
        this.supSdkProperties = supSdkProperties;
        this.applicationContext = applicationContext;
        createSDKInstance();
        callback = applicationContext.getBeansOfType(CmsServerCallback.class).values().stream().findFirst().orElse(null);
    }

    /**
     * 根据不同操作系统选择不同的库文件和库路径
     */
    private boolean createSDKInstance() {
        if (hciSupCms == null) {
            String libPath = supSdkProperties.libPath();
            synchronized (HCISUPCMS.class) {
                String strDllPath = "";
                try {
                    SystemUtil.jnaDebugLoad(supSdkProperties.getJnaDebug());
                    if (SystemUtil.isWindows()) {
                        strDllPath = libPath + "HCISUPCMS.dll";
                    } else if (SystemUtil.isLinux()) {
                        strDllPath = libPath + "libHCISUPCMS.so";
                    }
                    log.info("loadLibrary: {}", strDllPath);
                    hciSupCms = (HCISUPCMS) Native.loadLibrary(strDllPath, HCISUPCMS.class);
                } catch (Exception ex) {
                    log.error("loadLibrary: {}, Error: {}", strDllPath, ex.getMessage());
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * cms服务初始化，开启监听
     */
    private void cmsInit() {
        if (hciSupCms == null) {
            if (!createSDKInstance()) {
                throw new RuntimeException("Load CMS SDK fail!");
            }
        }
        String libPath = supSdkProperties.libPath();
        if (SystemUtil.isWindows()) {
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCrypto = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCrypto = libPath + "libeay32.dll"; //Linux版本是libcrypto.so库文件的路径
            System.arraycopy(strPathCrypto.getBytes(), 0, ptrByteArrayCrypto.byValue, 0, strPathCrypto.length());
            ptrByteArrayCrypto.write();
            hciSupCms.NET_ECMS_SetSDKInitCfg(0, ptrByteArrayCrypto.getPointer());

            //设置libssl.so所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArraySsl = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathSsl = libPath + "ssleay32.dll";    //Linux版本是libssl.so库文件的路径
            System.arraycopy(strPathSsl.getBytes(), 0, ptrByteArraySsl.byValue, 0, strPathSsl.length());
            ptrByteArraySsl.write();
            hciSupCms.NET_ECMS_SetSDKInitCfg(1, ptrByteArraySsl.getPointer());
            //注册服务初始化
            boolean binit = hciSupCms.NET_ECMS_Init();
            //设置HCAapSDKCom组件库文件夹所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCom = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCom = libPath + "HCAapSDKCom";        //只支持绝对路径，建议使用英文路径
            System.arraycopy(strPathCom.getBytes(), 0, ptrByteArrayCom.byValue, 0, strPathCom.length());
            ptrByteArrayCom.write();
            hciSupCms.NET_ECMS_SetSDKLocalCfg(5, ptrByteArrayCom.getPointer());

        } else if (SystemUtil.isLinux()) {
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCrypto = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCrypto = libPath + "libcrypto.so"; //Linux版本是libcrypto.so库文件的路径
            System.arraycopy(strPathCrypto.getBytes(), 0, ptrByteArrayCrypto.byValue, 0, strPathCrypto.length());
            ptrByteArrayCrypto.write();
            hciSupCms.NET_ECMS_SetSDKInitCfg(0, ptrByteArrayCrypto.getPointer());

            //设置libssl.so所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArraySsl = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathSsl = libPath + "libssl.so";    //Linux版本是libssl.so库文件的路径
            System.arraycopy(strPathSsl.getBytes(), 0, ptrByteArraySsl.byValue, 0, strPathSsl.length());
            ptrByteArraySsl.write();
            hciSupCms.NET_ECMS_SetSDKInitCfg(1, ptrByteArraySsl.getPointer());
            //注册服务初始化
            boolean binit = hciSupCms.NET_ECMS_Init();
            //设置HCAapSDKCom组件库文件夹所在路径
            HCISUPCMS.BYTE_ARRAY ptrByteArrayCom = new HCISUPCMS.BYTE_ARRAY(256);
            String strPathCom = libPath + "HCAapSDKCom/";        //只支持绝对路径，建议使用英文路径
            System.arraycopy(strPathCom.getBytes(), 0, ptrByteArrayCom.byValue, 0, strPathCom.length());
            ptrByteArrayCom.write();
            hciSupCms.NET_ECMS_SetSDKLocalCfg(5, ptrByteArrayCom.getPointer());

        }
        hciSupCms.NET_ECMS_SetLogToFile(3, supSdkProperties.logPath(), false);
    }

    /**
     * 开启监听
     */
    private void startCmsListen() {
        if (deviceRegisterCb == null) {
            Map<String, HCISUPCMS.DEVICE_REGISTER_CB> beansOfType = applicationContext.getBeansOfType(HCISUPCMS.DEVICE_REGISTER_CB.class);
            if (beansOfType.size() < 1) {
                throw new RuntimeException("not found device register callback implement!");
            }
            deviceRegisterCb = beansOfType.values().stream().findFirst().get();
        }
        System.arraycopy(supSdkProperties.getCmsServerIp().getBytes(), 0, struCMSListenPara.struAddress.szIP, 0, supSdkProperties.getCmsServerIp().length());
        struCMSListenPara.struAddress.wPort = Short.parseShort(supSdkProperties.getCmsServerPort().toString());
        struCMSListenPara.fnCB = deviceRegisterCb;
        struCMSListenPara.write();
        //启动监听，接收设备注册信息
        CmsHandle = hciSupCms.NET_ECMS_StartListen(struCMSListenPara);
        if (CmsHandle < -1) {
            log.info("NET_ECMS_StartListen failed, error code: {}", hciSupCms.NET_ECMS_GetLastError());
            hciSupCms.NET_ECMS_Fini();
            return;
        }
        String CmsListenInfo = new String(struCMSListenPara.struAddress.szIP).trim() + "_" + struCMSListenPara.struAddress.wPort;
        log.info("注册服务器: {}, NET_ECMS_StartListen succeed!\n", CmsListenInfo);
    }

    /**
     * 初始化CMS服务
     */
    void startCmsServer() {
        try {
            Optional<CmsServerCallback> opt = Optional.ofNullable(callback);
            opt.ifPresent(CmsServerCallback::preStart);
            cmsInit();
            startCmsListen();
            opt.ifPresent(CmsServerCallback::finishStart);
        } catch (Exception ex) {
            log.error("启动CMS注册服务失败: " + ex.getMessage(), ex);
        }
    }

    /**
     * 停止CMS服务,回收资源
     */
    void stopCmsServer() {
        try {
            if (CmsHandle >= 0) {
                log.info("开始停止CMS注册服务");
                hciSupCms.NET_ECMS_StopListen(CmsHandle);
                hciSupCms.NET_ECMS_Fini();
                log.info("停止CMS注册服务成功!");
                Optional.ofNullable(callback).ifPresent(CmsServerCallback::finishStop);
            }
        } catch (Exception ex) {
            log.error("停止CMS注册服务失败: " + ex.getMessage(), ex);
        }
    }

    /**
     * 添加人员工号信息
     *
     * @param loginID  登录设备句柄ID
     * @param userInfo 人员信息
     */
    public String addEmployeeInfo(int loginID, UserInfo userInfo) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "POST /ISAPI/AccessControl/UserInfo/Record?format=json"; //添加人员URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();

        //输入参数，XML或者JSON数据,下发人员信息json报文
        Map<String, Object> parameter = new HashMap<>();
        parameter.put("UserInfo", userInfo); // 员工ID
        String input = JSONObject.toJSONString(parameter);
        log.info("添加人员信息json报文: {}", input);
        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = 5000; //接收超时时间，单位毫秒
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("添加人员失败，NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\n输出报文: {}", res);
            return res;
        }
    }

    /**
     * 添加人员工号信息, 返回boolean结果
     *
     * @param loginID  登录设备句柄ID
     * @param userInfo 人员信息
     * @return true | false
     */
    public boolean addEmployeeInfoF(int loginID, UserInfo userInfo) {
        return Optional.ofNullable(addEmployeeInfo(loginID, userInfo)).isPresent();
    }

    /**
     * 删除人员工号信息
     *
     * @param loginID     登录设备句柄ID
     * @param employeeNos 员工编号
     */
    public String deleteEmployeeInfo(int loginID, String... employeeNos) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "PUT /ISAPI/AccessControl/UserInfoDetail/Delete?format=json"; //删除人员URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();

        //输入参数，XML或者JSON数据,删除人员信息json报文
        Map<String, Object> parameter = new HashMap<>();
        UserInfoDetail detail = new UserInfoDetail();
        detail.setMode("byEmployeeNo"); // 删除方式, byEmployeeNo: 按员工ID号
        List<UserInfoDetail.EmployeeNo> list = new ArrayList<>();
        for (String no : employeeNos) {
            list.add(new UserInfoDetail.EmployeeNo(no));
        }
        detail.setEmployeeNoList(list);
        parameter.put("UserInfoDetail", detail);

        String input = JSONObject.toJSONString(parameter);
        log.info("删除人员信息json报文: {}", input);
        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = 5000; //接收超时时间，单位毫秒
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("删除人员失败，NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\n删除人员成功, 输出报文: {}", res);
            return res;
        }
    }

    /**
     * 删除人员工号信息 返回boolean结果
     *
     * @param loginID     登录设备句柄ID
     * @param employeeNos 员工编号
     * @return true | false
     */
    public boolean deleteEmployeeInfoF(int loginID, String... employeeNos) {
        return Optional.ofNullable(deleteEmployeeInfo(loginID, employeeNos)).isPresent();
    }


    /**
     * 查询人员工号信息
     *
     * @param loginID 设备登录句柄ID
     */
    public String searchEmployeeInfo(int loginID) {
        return searchEmployeeInfo(loginID, 30);
    }

    /**
     * 查询人员工号信息
     *
     * @param loginID 登录设备句柄ID
     */
    public String searchEmployeeInfo(int loginID, int maxResults) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "POST /ISAPI/AccessControl/UserInfo/Search?format=json"; //查询人员URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();
        UUID uuid = UUID.randomUUID();

        //输入参数，XML或者JSON数据,查询多条人员信息json报文
        Map<String, Object> parameter = new HashMap<>();
        Map<String, Object> userInfoSearchCond = new HashMap<>();
        userInfoSearchCond.put("searchID", uuid); // 查询id
        userInfoSearchCond.put("searchResultPosition", 0);
        userInfoSearchCond.put("maxResults", 30); // 最大查询数量
        parameter.put("UserInfoSearchCond", userInfoSearchCond);
        String input = JSONObject.toJSONString(parameter);

        //输入参数，XML或者JSON数据,指定人员工号查询json报文
        log.info("查询人员工号信息json报文: {}", input);

        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = 5000; //接收超时时间，单位毫秒
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("查询人员失败，NET_ECMS_ISAPIPassThrough failed, error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed!\n查询人员成功, 输出报文: {}", res);
            return res;
        }
    }


    /**
     * 下发门禁人脸图片数据
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     * @param timeout 超时时间 单位ms
     */
    public String addFacePicInfo(int loginID, FaceInfoParam param, int timeout) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "POST /ISAPI/Intelligent/FDLib/FaceDataRecord?format=json"; //添加人脸URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();

        //输入参数，XML或者JSON数据,添加人脸图片json报文
        String input = JSONObject.toJSONString(param);
        log.info("下发门禁人脸图片 json报文: {}", input);

        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = timeout;
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("下发门禁人脸失败，NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\n下发门禁人脸成功, 输出报文: {}", res);
            return res;
        }
    }

    /**
     * 下发门禁人脸图片数据 默认5s
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     * @return 响应结果
     */
    public String addFacePicInfo(int loginID, FaceInfoParam param) {
        return addFacePicInfo(loginID, param, DEFAULT_TIMEOUT);
    }

    /**
     * 下发门禁人脸图片数据, 返回boolean结果
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     */
    public boolean addFacePicInfoF(int loginID, FaceInfoParam param, int timeout) {
        return Optional.ofNullable(addFacePicInfo(loginID, param, timeout)).isPresent();
    }

    /**
     * 下发门禁人脸图片数据, 返回boolean结果
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     */
    public boolean addFacePicInfoF(int loginID, FaceInfoParam param) {
        return addFacePicInfoF(loginID, param, DEFAULT_TIMEOUT);
    }

    /**
     * 修改门禁人脸图片数据
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     * @param timeout 超时 单位ms
     * @return 响应内容
     */
    public String modifyFacePicInfo(int loginID, FaceInfoParam param, int timeout) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        // 透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "PUT /ISAPI/Intelligent/FDLib/FDModify?format=json"; //修改人脸URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();

        //输入参数，XML或者JSON数据,添加人脸图片json报文
        String input = JSONObject.toJSONString(param);
        log.info("修改门禁人脸图片 json报文: {}", input);

        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = timeout; //接收超时时间，单位毫秒
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("修改门禁人脸失败，NET_ECMS_ISAPIPassThrough failed,error：" + hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\n修改门禁人脸成功, 输出报文: {}", res);
            return res;
        }
    }

    /**
     * 修改门禁人脸图片数据
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     * @return 响应内容
     */
    public String modifyFacePicInfo(int loginID, FaceInfoParam param) {
        return modifyFacePicInfo(loginID, param, DEFAULT_TIMEOUT);
    }

    /**
     * 修改门禁人脸图片数据, 返回boolean结果
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     */
    public boolean modifyFacePicInfoF(int loginID, FaceInfoParam param, int timeout) {
        return Optional.ofNullable(modifyFacePicInfo(loginID, param, timeout)).isPresent();
    }

    /**
     * 修改门禁人脸图片数据, 返回boolean结果
     *
     * @param loginID 登录设备句柄ID
     * @param param   人脸数据
     */
    public boolean modifyFacePicInfoF(int loginID, FaceInfoParam param) {
        return modifyFacePicInfoF(loginID, param, DEFAULT_TIMEOUT);
    }

    /**
     * 删除人脸图片数据
     *
     * @param loginID    登录设备句柄ID
     * @param employeeNo 人员工号，按照指定人员删除关联人脸图片
     * @return 响应结果
     */
    public String deleteFacePicInfo(int loginID, String employeeNo, int timeout) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "PUT /ISAPI/Intelligent/FDLib/FDSearch/Delete?format=json&FDID=1&faceLibType=blackFD"; //删除人脸URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();

        //输入参数，XML或者JSON数据,删除人脸图片json报文
        Map<String, Object> parameter = new HashMap<>();
        List<Map<String, Object>> list = new ArrayList<>();
        Map<String, Object> map = new HashMap<>();
        map.put("value", employeeNo);
        list.add(map);
        parameter.put("FPID", list);

        String input = JSONObject.toJSONString(parameter);
        log.info("删除人脸图片数据 json报文: {}", input);

        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = timeout; //接收超时时间，单位毫秒
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("删除门禁人脸失败，NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\n删除门禁人脸成功, 输出报文: {}", res);
            return res;
        }
    }

    /**
     * 删除人脸图片数据
     *
     * @param loginID    登录设备句柄ID
     * @param employeeNo 人员工号，按照指定人员删除关联人脸图片
     * @return 响应结果
     */
    public String deleteFacePicInfo(int loginID, String employeeNo) {
        return deleteFacePicInfo(loginID, employeeNo, DEFAULT_TIMEOUT);
    }

    /**
     * 删除人脸图片数据
     *
     * @param loginID    登录设备句柄ID
     * @param employeeNo 人员工号，按照指定人员删除关联人脸图片
     */
    public boolean deleteFacePicInfoF(int loginID, String employeeNo, int timeout) {
        return Optional.ofNullable(deleteFacePicInfo(loginID, employeeNo, timeout)).isPresent();
    }

    /**
     * 删除人脸图片数据
     *
     * @param loginID    登录设备句柄ID
     * @param employeeNo 人员工号，按照指定人员删除关联人脸图片
     */
    public boolean deleteFacePicInfoF(int loginID, String employeeNo) {
        return deleteFacePicInfoF(loginID, employeeNo, DEFAULT_TIMEOUT);
    }

    /**
     * 查询门禁人脸图片
     *
     * @param loginID    登录设备句柄ID
     * @param employeeNo 人员工号，按照指定人员查询关联人脸图片
     * @param timeout    超时 单位ms
     * @return 响应结果
     */
    public String searchFacePicInfo(int loginID, String employeeNo, int timeout) {
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();
        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String urlInBuffer = "POST /ISAPI/Intelligent/FDLib/FDSearch?format=json"; //查询人脸URL
        HCISUPCMS.BYTE_ARRAY ptrurlInBuffer = new HCISUPCMS.BYTE_ARRAY(urlInBuffer.length() + 1);
        System.arraycopy(urlInBuffer.getBytes(), 0, ptrurlInBuffer.byValue, 0, urlInBuffer.length());
        ptrurlInBuffer.write();
        m_struParam2.pRequestUrl = ptrurlInBuffer.getPointer();
        m_struParam2.dwRequestUrlLen = urlInBuffer.length();

        //输入参数，XML或者JSON数据,查询人脸图片json报文
        SearchFaceInfoParam param = new SearchFaceInfoParam(employeeNo);
        String input = JSONObject.toJSONString(param);
        //批量查询人脸图片json报文
        log.info("查询门禁人脸图片 json报文: {}", input);

        byte[] byInbuffer = input.getBytes(StandardCharsets.UTF_8);
        int iInBufLen = byInbuffer.length;
        HCISUPCMS.BYTE_ARRAY ptrInBuffer = new HCISUPCMS.BYTE_ARRAY(iInBufLen);
        ptrInBuffer.read();
        System.arraycopy(byInbuffer, 0, ptrInBuffer.byValue, 0, iInBufLen);
        ptrInBuffer.write();
        m_struParam2.pInBuffer = ptrInBuffer.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInBufLen;
        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;
        m_struParam2.dwRecvTimeOut = 5000; //接收超时时间，单位毫秒
        m_struParam2.write();
        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginID, m_struParam2)) {
            log.info("查询门禁人脸失败，NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\n查询门禁人脸成功, 输出报文: {}", res);
            return res;
        }
    }

    /**
     * 查询门禁人脸图片
     *
     * @param loginID    登录设备句柄ID
     * @param employeeNo 人员工号，按照指定人员查询关联人脸图片
     * @return 响应结果
     */
    public String searchFacePicInfo(int loginID, String employeeNo) {
        return searchFacePicInfo(loginID, employeeNo, DEFAULT_TIMEOUT);
    }

    /**
     * ISUP透传接口 [云存储配置]
     */
    public String setPictureServer(int loginId, String paramStr) {
        /*
         * GET方法示例（以获取云存储配置为例）
         */
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam.read();

        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String url = "GET /ISAPI/System/PictureServer?format=json"; //获取云存储配置
        HCISUPCMS.BYTE_ARRAY ptrUrl = new HCISUPCMS.BYTE_ARRAY(url.length() + 1);
        System.arraycopy(url.getBytes(), 0, ptrUrl.byValue, 0, url.length());
        ptrUrl.write();
        m_struParam.pRequestUrl = ptrUrl.getPointer();
        m_struParam.dwRequestUrlLen = url.length();

        //输入参数，XML或者JSON数据
        m_struParam.pInBuffer = null;//GET获取时不需要输入参数，输入为null
        m_struParam.dwInSize = 0;

        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte = new HCISUPCMS.BYTE_ARRAY(iOutSize);
        m_struParam.pOutBuffer = ptrOutByte.getPointer();
        m_struParam.dwOutSize = iOutSize;

        m_struParam.dwRecvTimeOut = 5000; //接收超时时间，单位毫秒
        m_struParam.write();

        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginId, m_struParam)) {
            log.info("NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam.read();
            ptrOutByte.read();
            log.info("NET_ECMS_ISAPIPassThrough succeed\nptrOutByte: {}", new String(ptrOutByte.byValue).trim());
        }

        /*
         * PUT方法示例（以设置云存储配置为例）
         */
        HCISUPCMS.NET_EHOME_PTXML_PARAM m_struParam2 = new HCISUPCMS.NET_EHOME_PTXML_PARAM();
        m_struParam2.read();

        //透传URL，不同功能对应不同的URL，完整协议报文说明需要参考ISAPI协议文档
        String url2 = "PUT /ISAPI/System/PictureServer?format=json"; //获取云存储配置
        HCISUPCMS.BYTE_ARRAY ptrUrl2 = new HCISUPCMS.BYTE_ARRAY(url2.length() + 1);
        System.arraycopy(url2.getBytes(), 0, ptrUrl2.byValue, 0, url2.length());
        ptrUrl2.write();
        m_struParam2.pRequestUrl = ptrUrl2.getPointer();
        m_struParam2.dwRequestUrlLen = url2.length();

        //输入参数，XML或者JSON数据
//        Map<String, Object> parameter = new HashMap<>();
//        parameter.put("ipv4Address", propertiesUtil.readValue("PicServerIP"));
//        parameter.put("portNo", Short.parseShort(propertiesUtil.readValue("PicServerPort")));
//        String input = ConfigFileUtil.getReqBodyFromTemplate("conf/SdkCms/PutPicServerParam.json", parameter);
//        System.out.println(input);

        //String input = "{\"PictureServerInformation\":{\"pictureServerType\":\"cloudStorage\",\"addressingFormatType\":\"ipaddress\",\"hostName\":\"\",\"ipv4Address\":\"118.195.144.241\",\"ipv6Address\":\"0\",\"portNo\":6011,\"cloudStorage\":{\"cloudManageHttpPort\":6011,\"cloudTransDataPort\":6011,\"cloudCmdPort\":6011,\"cloudHeartBeatPort\":6011,\"cloudStorageHttpPort\":6011,\"cloudUsername\":\"test\",\"cloudPassword\":\"12345\",\"cloudPoolId\":1,\"clouldProtocolVersion\":\"cloud2\",\"clouldAccessKey\":\"test\",\"clouldSecretKey\":\"12345\"}}}";
        int iInSize = paramStr.getBytes(StandardCharsets.UTF_8).length;
        HCISUPCMS.BYTE_ARRAY ptrInParam = new HCISUPCMS.BYTE_ARRAY(iInSize + 1);
        System.arraycopy(paramStr.getBytes(StandardCharsets.UTF_8), 0, ptrInParam.byValue, 0, iInSize);
        ptrInParam.write();
        m_struParam2.pInBuffer = ptrInParam.getPointer();//GET获取时不需要输入参数，输入为null
        m_struParam2.dwInSize = iInSize;

        //输出参数，分配的内存用于存储返回的数据，需要大于等于实际内容大小
        int iOutSize2 = 2 * 1024 * 1024;
        HCISUPCMS.BYTE_ARRAY ptrOutByte2 = new HCISUPCMS.BYTE_ARRAY(iOutSize2);
        m_struParam2.pOutBuffer = ptrOutByte2.getPointer();
        m_struParam2.dwOutSize = iOutSize2;

        m_struParam2.dwRecvTimeOut = 5000; //接收超时时间，单位毫秒
        m_struParam2.write();

        if (!hciSupCms.NET_ECMS_ISAPIPassThrough(loginId, m_struParam2)) {
            log.error("NET_ECMS_ISAPIPassThrough failed,error：{}", hciSupCms.NET_ECMS_GetLastError());
            return null;
        } else {
            m_struParam2.read();
            ptrOutByte2.read();
            String res = new String(ptrOutByte2.byValue).trim();
            log.info("NET_ECMS_ISAPIPassThrough succeed\nptrOutByte: {}", res);
            return res;
        }
    }

}
